unit PICConnector;

interface

uses Windows, Dialogs, Sysutils,usbdll;

const
  SET_PIC_LED='0'; //$30;
  GET_PIC_TEMP=$54;//'T';
  GET_PIC_LIGHT=$4C;//'L';


type
  TPICConnector = class(TObject)
  private
    { Private declarations }
    pga_range: byte;
  public
    { Public declarations }
    constructor Open;
    destructor Close;
    function SetPicLed(an: Boolean):Boolean;
    function GetPicTemperature: Single;
    function GetPicLight: LongInt;
    function GetPgaRange: LongInt;
  end;

implementation

constructor TPICConnector.Open;
var count: DWORD;
begin
  count := _MPUSBGetDeviceCount(vid_pid);
  if (count > 0) then
	begin
		myOutPipe := _MPUSBOpen(0,vid_pid,out_pipe,MP_WRITE,0);
		myInPipe  := _MPUSBOpen(0,vid_pid,in_pipe,MP_READ,0);
		if (myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE) then
			ShowMessage('Failed to open data pipes.')
	end
	else
		ShowMessage('No devices connected.');
end;

destructor TPICConnector.Close;
begin
	if (myOutPipe <> INVALID_HANDLE_VALUE) then _MPUSBClose(myOutPipe);
  if (myInPipe <> INVALID_HANDLE_VALUE) then _MPUSBClose(myInPipe);
  myOutPipe := INVALID_HANDLE_VALUE;
  myInPipe := INVALID_HANDLE_VALUE;
end;


function TPICConnector.SetPicLed(an: Boolean): Boolean;
var
  send_buf, receive_buf :PBYTE;
  RecvLength: LongInt;
begin
  RecvLength:= 1;
	send_buf[0] := Ord(SET_PIC_LED);
	if (an) then
		send_buf[1] := 1
	else
		send_buf[1] := 0;

	if (SendReceivePacket(send_buf, 2, receive_buf, RecvLength, 100, 100) = 1) then
  begin
		if (RecvLength <> 1) or (receive_buf[0] <> Ord(SET_PIC_LED)) then
    begin
    	ShowMessage('PIC-Fehler #1');
			result := false;
      exit;
		end
	end
	else
	begin
		ShowMessage('USB-Fehler #1');
		result := false;
    exit;
	end;
	Result := true;

end;

function TPICConnector.GetPicTemperature;
var
  send_buf, receive_buf : PBYTE;
  RecvLength: LongInt;
  temp: single;
begin
  RecvLength := 2;
	send_buf[0] := GET_PIC_TEMP;
	if (SendReceivePacket(send_buf, 1, receive_buf, RecvLength, 100, 100) = 1) then
	begin
		if (RecvLength <> 2) then
    begin
			ShowMessage('PIC-Fehler #2');
			Result := 0;
      exit;
		end
	end
	else
	begin
		ShowMessage('USB-Fehler #1');
		Result := 0;
    exit;
	end;
  temp := 256 * receive_buf[1] + receive_buf[0];
  temp := temp / 8;
	temp := 625 * temp / 1000;
  temp := temp / 10.0;
	Result :=  temp
end;

function TPICConnector.GetPgaRange;
begin
  Result := pga_range;
end;

function TPICConnector.GetPicLight;
var
  send_buf, receive_buf: PBYTE;
  RecvLength: LongInt;
begin
  RecvLength := 2;
  send_buf[0] := GET_PIC_LIGHT;
	if (SendReceivePacket(send_buf, 1, receive_buf, RecvLength, 100, 100) = 1) then
	begin
		if (RecvLength <> 3) then
		begin
			ShowMessage('PIC-Fehler #3');
			Result := 0;
      exit;
		end
	end
	else
	begin
		ShowMessage('USB-Fehler #1');
		Result := 0;
    exit;
	end;
	pga_range := receive_buf[2];
	Result := 256 * receive_buf[1] + receive_buf[0];
end;


end.
