// Version 1.0
// Compiler Borland Developer Studio 2006
// This software is free for non commercial use,
// for commercial use, please contact the author.

unit PICUSBClass;

interface

uses
  SysUtils, Classes, usbAPI, windows;
Const vid_pid  : PCHAR  = 'vid_04d8&pid_000c' + #0;    // Default Demo Application Firmware
      out_pipe : PCHAR = '\MCHP_EP1' + #0;
      in_pipe  : PCHAR = '\MCHP_EP1' + #0;

      READ_VERSION      = $00;
      WRITE_READ_DATA   = $04;
      SET_LED           = $06;
      SET_PIC_LED       = $30; //'0'
      GET_PIC_TEMP      = $54; //'T'
      GET_PIC_LIGHT     = $4C; //'L'

      UsbBufSize = 64;

type
  PByteBuffer = ^TByteBuffer;
  TByteBuffer = Array[0..63] of Byte;

  TPICUSB = class(TComponent)
  private
    { Private declarations }
    myOutPipe : THANDLE;
    myInPipe  : THANDLE;
    isActive  : Boolean;
    isLedOn   : Boolean;
    pgaRange  : Longint;
    SndDelay  : Word;
    RcvDelay  : Word;
    procedure setActive(Active: Boolean);
    function getTemperature: Single;
    function getLight: Longint;
    function getRange: Longint;
    function getVersion: String;
    procedure SetLed(LedOn: Boolean);
    function getLedStatus: Boolean;
    procedure setSendDelay(Value: Word);
    function getSendDelay: Word;
    procedure setRcvDelay(Value: Word);
    function getRcvDelay: Word;
    Function SendReceivePacket(SendData :PByteBuffer; SendLength : DWORD ; ReceiveData : PByteBuffer; var ReceiveLength : DWORD):Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Active: Boolean read isActive write setActive;
    property Temperature: Single read getTemperature;
    property Light: Longint read getLight;
    property Range: Longint read getRange;
    property Version: String read getVersion;
    property LedOn: Boolean read getLedStatus write SetLed;
    property SendDelay: Word read getSendDelay write setSendDelay;
    property ReceiveDelay: Word read getRcvDelay write setRcvDelay;
  end;

{procedure Register; }

implementation

Constructor TPICUSB.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  isActive:= False;
  myOutPipe := INVALID_HANDLE_VALUE;
  myInPipe := INVALID_HANDLE_VALUE;
  isLedOn := False;
  pgaRange := 0;
  SndDelay := 1000;
  RcvDelay := 1000;
end;

Destructor TPICUSB.Destroy;
begin
  if isActive then
  begin
    MPUSBClose(myOutPipe);
    MPUSBClose(myInPipe);
    myOutPipe := INVALID_HANDLE_VALUE;
    myInPipe := INVALID_HANDLE_VALUE;
  end;
  inherited;
end;

procedure TPICUSB.setActive(Active: Boolean);
Var Selection : DWORD;
begin
  if Active then begin
    Selection:=0; // according to our firmware, only index 0 is available!
    myOutPipe := MPUSBOpen(selection,vid_pid, out_pipe, MP_WRITE, 0);
    myInPipe  := MPUSBOpen(selection,vid_pid, in_pipe,  MP_READ,  0);
  end else
  begin
    MPUSBClose(myOutPipe);
    MPUSBClose(myInPipe);
    myOutPipe := INVALID_HANDLE_VALUE;
    myInPipe := INVALID_HANDLE_VALUE;
  end;
    If (myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE) then
      isActive := False
    else
      isActive := True;
end;

function TPICUSB.getTemperature: single;
var raw: word;
  temp: smallint;
  RecvLength  : DWORD;
  send_buf    : TByteBuffer;
  receive_buf : TByteBuffer;
begin
  send_buf[0] := GET_PIC_TEMP;
  RecvLength := 2;
  if SendReceivePacket(@send_buf,1,@receive_buf,RecvLength) Then
  Begin
    raw := 256 * receive_buf[1] + receive_buf[0];
    temp := raw;
    temp := temp SHR 3;// remove status bits ?!
    temp := 625 * temp DIV 1000;   // Umrechnung von 1/16 zu 1/10
    result := temp DIV 10 + 0.1*(temp mod 10);

  end else
  begin
    result := -273.15;
    setActive(False);
  end;
end;

function TPICUSB.getLight: Longint;
var
  RecvLength  : DWORD;
  send_buf    : TByteBuffer;
  receive_buf : TByteBuffer;
begin
  send_buf[0] := GET_PIC_LIGHT;
  RecvLength := 3;
  if SendReceivePacket(@send_buf,1,@receive_buf,RecvLength) Then
  Begin
    pgaRange := receive_buf[2];
    result := 256 * receive_buf[1] + receive_buf[0];
  end else
  begin
    pgaRange := 0;
    result := 0;
    setActive(False);
  end;

end;

function TPICUSB.getLedStatus;
begin
  result := isLedOn;
end;

function TPICUSB.getRange;
begin
  result := pgaRange;
end;

function TPICUSB.getSendDelay;
begin
  result := SndDelay;
end;

function TPICUSB.getRcvDelay;
begin
  result := RcvDelay;
end;

procedure TPICUSB.setSendDelay(Value: Word);
begin
  SndDelay := Value;
end;

procedure TPICUSB.setRcvDelay(Value: Word);
begin
  RcvDelay := Value;
end;

procedure TPICUSB.SetLed(LedOn: Boolean);
var
  RecvLength  : DWORD;
  send_buf    : TByteBuffer;
  receive_buf : TByteBuffer;
begin
  send_buf[0] := SET_PIC_LED;
  if LedOn then send_buf[1] := 1 else send_buf[0] := 0;
  RecvLength := 2;
  if SendReceivePacket(@send_buf,2,@receive_buf,RecvLength) Then
  Begin
    isLedOn := LedOn;
  end else
  begin
    isLedOn := False;
    setActive(False);
  end;
end;

function TPICUSB.getVersion:String;
var temp:DWORD;
    versionInfo : Array[0..3] of Byte;
begin
  temp := MPUSBGetDLLVersion();
  move(temp,VersionInfo,sizeof(VersionInfo));
  result := intTOStr(VersionInfo[0]) +  '.' + intTOStr(VersionInfo[1]) +  '.' + intTOStr(VersionInfo[2]) +  '.'+ intTOStr(VersionInfo[3]);
end;

Function TPICUSB.SendReceivePacket(SendData :PByteBuffer; SendLength : DWORD ; ReceiveData : PByteBuffer; var ReceiveLength : DWORD):Boolean;
var
   SentDataLength:  DWORD ;
   ExpectedReceiveLength: DWORD;
Begin
  ExpectedReceiveLength := ReceiveLength;
  if(myOutPipe <> INVALID_HANDLE_VALUE) and ( myInPipe <> INVALID_HANDLE_VALUE) then
  Begin
    if MPUSBWrite(myOutPipe,SendData,SendLength,@SentDataLength,SendDelay) <> 0 then
    Begin
      if(MPUSBRead(myInPipe,ReceiveData, ExpectedReceiveLength,@ReceiveLength,ReceiveDelay)) <> 0 then
      Begin
        if (ReceiveLength = ExpectedReceiveLength) Then
        Begin
          result:=True;   // Success!
          Exit;
        End
      End //Read
    End //Write
  End; //invalid_Handle
  result:=False;  // Operation Failed
  setActive(False);
End;//end SendReceivePacket

{
procedure Register;
begin
  RegisterComponents('Samples', [TPICUSB]);
end;
}
end.
