unit UMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, PICUSBClass, ComCtrls, TeEngine, Series, ExtCtrls,
  TeeProcs, Chart, DateUtils, Tabs, UShutdown, Math, Mask, JvExMask, JvToolEdit,
  JvBaseEdits, IniFiles;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    MIOpen: TMenuItem;
    MIClose: TMenuItem;
    N1: TMenuItem;
    MIExit: TMenuItem;
    SBInfo: TStatusBar;
    PageControl1: TPageControl;
    TSPlot: TTabSheet;
    TSData: TTabSheet;
    Chart1: TChart;
    Series1: TFastLineSeries;
    Memo1: TMemo;
    Plot1: TMenuItem;
    MILogarithmic: TMenuItem;
    MIClear: TMenuItem;
    Series2: TFastLineSeries;
    TSSettings: TTabSheet;
    LSendDelay: TLabel;
    LReceiveDelay: TLabel;
    TBSendDelay: TTrackBar;
    TBReceiveDelay: TTrackBar;
    GBCalibration: TGroupBox;
    LFormula: TLabel;
    LParamA: TLabel;
    LParamB: TLabel;
    LParamC: TLabel;
    LBreakpoint: TLabel;
    CBReboot: TCheckBox;
    TabSet1: TTabSet;
    TReboot: TTimer;
    Series3: TFastLineSeries;
    Series4: TFastLineSeries;
    Series5: TFastLineSeries;
    Series6: TFastLineSeries;
    CEParamA: TJvCalcEdit;
    CEParamB: TJvCalcEdit;
    CEParamC: TJvCalcEdit;
    CEBreakPoint: TJvCalcEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    LLocationID: TLabel;
    LLongitude: TLabel;
    LLatitude: TLabel;
    LHeight: TLabel;
    LType: TLabel;
    LID: TLabel;
    LDescription: TLabel;
    LContact: TLabel;
    LAddress: TLabel;
    Lemail: TLabel;
    ELocationID: TEdit;
    CELongitude: TJvCalcEdit;
    CELatitude: TJvCalcEdit;
    EDescription: TEdit;
    EAddress: TEdit;
    EID: TEdit;
    EContact: TEdit;
    CBType: TComboBox;
    Eemail: TEdit;
    CEHeight: TJvCalcEdit;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CEParamEnter(Sender: TObject);
    procedure CEParamExit(Sender: TObject);
    procedure TRebootTimer(Sender: TObject);
    procedure TabSet1Change(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure TBDelayChange(Sender: TObject);
    procedure MIClearClick(Sender: TObject);
    procedure MILogarithmicClick(Sender: TObject);
    procedure IdleHandler(Sender: TObject; var Done: Boolean);
    procedure SBInfoResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MIExitClick(Sender: TObject);
  private
    { Private declarations }
    MySensor: TPICUSB;
    SensorStatus: string;
    TimeStamp: TDateTime;
    Temperature: Single;
    Flux,
    LastSecond: LongInt;
    Filename: TFilename;
    data: array [0..86399] of double;
    ndata: array [0..86399] of double;
    a,b,c,breakpoint: double;
    IniFile: TInifile;
    procedure UpdateStatusbar;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.UpdateStatusbar;
begin
  if Form1.Visible then begin
    SBInfo.Panels[1].Text := 'Status: '+Sensorstatus;
    SBInfo.Panels[2].Text := FormatDateTime('yyyy-mm-dd hh:nn:ss.zzz',TimeStamp);
    if Temperature <= -273.15 then SBInfo.Panels[4].Text := 'Temperature: n/a' else
      SBInfo.Panels[3].Text := Format('Temperature: %5.1f C',[Temperature]);
    if Flux = 0 then SBInfo.Panels[4].Text := 'Flux: n/a' else
      SBInfo.Panels[4].Text := 'Flux: '+IntToStr(Round(Flux))+' counts';
  end;
end;

procedure TForm1.CEParamEnter(Sender: TObject);
begin
  CEParamA.Value := a/1e-9;
  CEParamB.Value := b/1e-6;
  CEParamC.Value := c/1e3;
  CEBreakpoint.Value := Breakpoint;
end;

procedure TForm1.CEParamExit(Sender: TObject);
begin
  a := CEParamA.Value*1e-9;
  b := CEParamB.Value*1e-6;
  c := CEParamC.Value*1e3;
  Breakpoint := CEBreakpoint.Value;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with Inifile do
  begin
    WriteInteger('Parameters','a',CEParamA.AsInteger);
    WriteInteger('Parameters','b',CEParamB.AsInteger);
    WriteInteger('Parameters','c',CEParamC.AsInteger);
    WriteInteger('Parameters','Breakpoint',CEBreakpoint.AsInteger);
    WriteInteger('Sensor','SendDelay',MySensor.SendDelay);
    WriteInteger('Sensor','ReceiveDelay',MySensor.ReceiveDelay);
    WriteBool('General','Reboot',CBReboot.Checked);
    WriteString('General','LocationID',ELocationID.Text);
    WriteString('General','Longitude',CELongitude.Text);
    WriteString('General','Latitude',CELatitude.Text);
    WriteString('General','Height',CEHeight.Text);
    WriteString('General','Description',EDescription.Text);
    WriteString('General','Type',CBType.Text);
    WriteString('General','ID',EID.Text);
    WriteString('General','Contact',EContact.Text);
    WriteString('General','Address',EAddress.Text);
    WriteString('General','Email',Eemail.Text);
    Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var inifilename: TFilename;
begin
  Flux := 0;
  Temperature := -1000;
  TimeStamp := now;
  a := 2.48e-7;
  b := 0.7e-5;
  c := 1.0e5;
  Breakpoint := 754717.0;

  inifilename := ExtractFilePath(ParamStr(0))+'Skysensor.ini';
  IniFile := TIniFile.Create(inifilename);
  With Inifile do begin
    CEParamA.Value := ReadInteger('Parameters','a',248);
    CEParamB.Value := ReadInteger('Parameters','b',7);
    CEParamC.Value := ReadInteger('Parameters','c',100);
    CEBreakpoint.Value := ReadInteger('Parameters','Breakpoint',754717);
    CBReboot.Checked := ReadBool('General','Reboot',False);
    ELocationID.Text := ReadString('General','LocationID','');
    CELongitude.Text := ReadString('General','Longitude','');
    CELatitude.Text := ReadString('General','Latitude','');
    CEHeight.Text := ReadString('General','Height','');
    EDescription.Text := ReadString('General','Description','');
    CBType.Text := ReadString('General','Type','');
    EID.Text := ReadString('General','ID','');
    EContact.Text := ReadString('General','Contact','');
    EAddress.Text := ReadString('General','Address','');
    Eemail.Text := ReadString('General','Email','');
  end;
  CEParamExit(Sender);

  Filename := FormatDateTime('yyyymmdd',TimeStamp)+'.txt';
  if FileExists(Filename) then Memo1.Lines.LoadFromFile(Filename);
  MySensor := TPICUSB.Create(Form1);
  SBInfo.Panels[0].Text := 'DLL Version: '+ MySensor.Version;
  MySensor.Active := True;
  MySensor.SendDelay := IniFile.ReadInteger('Sensor','Senddelay',50);
  MySensor.ReceiveDelay := IniFile.ReadInteger('Sensor','Receivedelay',50);
  {  if MySensor.Active then Application.OnIdle := IdleHandler else
    Application.OnIdle := nil;  }
  if MySensor.Active then SensorStatus := 'Ok' else SensorStatus := 'Error';
  PageControl1.TabIndex := 0;
  Application.OnIdle := IdleHandler;
  Series1.Clear;
  Series2.Clear;
  Series3.Clear;
  Series4.Clear;
  Series5.Clear;
  Series6.Clear;
  LastSecond := -1;

end;

procedure TForm1.IdleHandler(Sender: TObject; var Done: Boolean);
var Light,
    avgFlux,
    Range: Longint;
    lux, mag, limmag: Extended;
    i: longint;
    Value: String;
begin
  TimeStamp := now;
  i := SecondOfTheDay(Timestamp);
  Flux := 0;
  Range := 0;
  Light := 0;
  lux := 0;
  mag := 0;
  limmag := 0;
//  Flux := Round(random()*2040360.0);
//  Temperature := random()*50.0-20.0;


  if MySensor.Active then begin
    TimeStamp := now;
    Light := MySensor.Light;
    Range := MySensor.Range;
    case Range of
      1: Flux := Light * 120;
      2: Flux := Light * 8;
      3: Flux := Light * 4;
      4: Flux := Light * 2;
      5: Flux := Light * 1;
    else
      Flux := 0;
    end;
    data[i] := data[i] + Flux;
    ndata[i] := ndata[i]+1.;
    Temperature := MySensor.Temperature;
    //Series1.Add(Flux);

  end;
  if (i<>Lastsecond) and (ndata[LastSecond] > 0) then begin
    avgFlux := Round(data[Lastsecond]/ndata[Lastsecond]);
    if avgFlux<Breakpoint then
      lux := a*avgFlux
    else
      if c > 0.0 then lux := a*avgflux+b*exp(avgflux/c);
    if lux>0.0 then mag := -2.5*Log10(lux)+15.2;
    limmag := mag - 16.0;
    if Lastsecond > 0 then data[Lastsecond] := 0;
    if Lastsecond > 0 then ndata[Lastsecond] := 0;
    Filename := FormatDateTime('yyyymmdd',TimeStamp)+'.txt';
    Memo1.Lines.Add(FormatDateTime('yyyy-mm-dd hh:nn:ss',TimeStamp)+Format(' %05.1f %07d %1d',[Temperature,avgFlux,Range]));

    if (Lastsecond mod 60) = 29 then Memo1.Lines.SaveToFile(Filename);

    if (Lastsecond=86398) or (i < LastSecond) then begin
      Memo1.Lines.SaveToFile(Filename);
      Memo1.Lines.Clear;
      Series1.Clear;
      Series2.Clear;
      Series3.Clear;
      Series4.Clear;
      Series5.Clear;
      Series6.Clear;
    end;
    //UpdateChart;
    if MySensor.Active then begin
      Value :=  FormatDateTime('hh:nn:ss',TimeStamp);
      Series1.AddXY(i,avgFlux,Value);
      Series2.AddXY(i,Temperature,Value);
      Series3.AddXY(i,Range,Value);
      Series4.AddXY(i,lux,Value);
      Series5.AddXY(i,mag,Value);
      Series6.AddXY(i,limmag,Value);

    end
    else if CBReboot.Checked then TReboot.Enabled := True;



  end;
//  if i<>Lastsecond then Series1.AddXY(LastSecond,ndata[Lastsecond]);
  LastSecond := i;
  if MySensor.Active then SensorStatus := 'Ok' else SensorStatus := 'Error';
  UpdateStatusBar;
  if not MySensor.Active then MySensor.Active := True;

  Done := False;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Memo1.Lines.SaveToFile(Filename);
  MySensor.Destroy;
end;

procedure TForm1.MIClearClick(Sender: TObject);
begin
  Series1.Clear;
end;

procedure TForm1.MIExitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.MILogarithmicClick(Sender: TObject);
begin
  MILogarithmic.Checked := not MILogarithmic.Checked;
  Chart1.LeftAxis.Logarithmic := MILogarithmic.Checked;
end;

procedure TForm1.SBInfoResize(Sender: TObject);
begin
  SBInfo.Panels[0].Width := SBInfo.Width DIV 5;
  SBInfo.Panels[1].Width := SBInfo.Width DIV 5;
  SBInfo.Panels[2].Width := SBInfo.Width DIV 5;
  SBInfo.Panels[3].Width := SBInfo.Width DIV 5;
  SBInfo.Panels[4].Width := SBInfo.Width DIV 5;
end;

procedure TForm1.TabSet1Change(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
  Series1.Active := False;   // counts
  Series2.Active := False;   // Temp
  Series3.Active := False;   // Range
  Series4.Active := False;   // lux
  Series5.Active := False;   // mag
  Series6.Active := False;   // lim mag
  case NewTab of
    0: Series1.Active := True;
    1: Series2.Active := True;
    2: Series3.Active := True;
    3: Series4.Active := True;
    4: Series5.Active := True;
    5: Series6.Active := True;
  end;
  case NewTab of
    0: Chart1.LeftAxis.Increment := 1e3;
    1: Chart1.LeftAxis.Increment := 0.1;
    2: Chart1.LeftAxis.Increment := 1.0;
    3: Chart1.LeftAxis.Increment := 1e3;
    4: Chart1.LeftAxis.Increment := 0.1;
    5: Chart1.LeftAxis.Increment := 0.1;
  end;
  Chart1.LeftAxis.Title.Caption := TabSet1.Tabs.Strings[NewTab];

end;

procedure TForm1.TBDelayChange(Sender: TObject);
begin
  MySensor.SendDelay := TBSendDelay.Position;
  LSendDelay.Caption := 'Send Delay: '+IntToStr(MySensor.SendDelay)+'ms';
  MySensor.ReceiveDelay := TBReceiveDelay.Position;
  LReceiveDelay.Caption := 'Receive Delay: '+IntToStr(MySensor.ReceiveDelay)+'ms';
end;

procedure TForm1.TRebootTimer(Sender: TObject);
begin
  WindowsExit(EWX_REBOOT or EWX_FORCE);
end;

end.
